@@ -1,12 +1,11 @@
 ;;; pod-mode.el --- Major mode for editing .pod-files
 
-;;; It mainly defines a grammar for syntax highlighting.
 ;;; POD is the Plain Old Documentation format of Perl.
 
-;;; Copyright 2003-2008 Steffen Schwigon
+;;; Copyright 2003-2009 Steffen Schwigon
 
 ;;; Author: Steffen Schwigon <ss5@renormalist.net>
-;;; Version: 0.501
+;;; Version: 1.00
 ;;; Keywords: perl pod
 ;;; X-URL: http://search.cpan.org/~schwigon/pod-mode/
 
@@ -144,6 +143,46 @@
                       ))))
   )
 
+(defun pod-enable-weaver-collector-keywords (collectors)
+  (let* ((keyword-list (mapcar (lambda (collector)
+                                 (symbol-name (getf collector 'command)))
+                               collectors))
+         (keyword-re (concat "^=" (regexp-opt keyword-list t))))
+    (setf pod-font-lock-keywords
+          (append pod-font-lock-keywords
+                  `((,keyword-re 0 font-lock-keyword-face))
+                  `((,(concat keyword-re "\\(.*\\)") 2 font-lock-comment-face))))
+    (setq font-lock-mode-major-mode nil)
+    (font-lock-fontify-buffer)))
+
+(defun pod-enable-weaver-features (weaver-config)
+  (pod-enable-weaver-collector-keywords (getf weaver-config 'collectors))
+  (message "Pod::Weaver keywords loaded."))
+
+(defvar pod-weaver-config-buffer "")
+
+(defun pod-load-weaver-config (dir)
+  "Load additional pod keywords from a projects dist.ini/weaver.ini"
+  (let* ((proc (start-process-shell-command
+                (concat "weaverconf-" (buffer-name (current-buffer)))
+                nil (format "cd %s; dzil weaverconf -f lisp" dir))))
+    (make-local-variable 'pod-weaver-config-buffer)
+    (set-process-filter
+     proc (lambda (proc str)
+            (setq pod-weaver-config-buffer (concat pod-weaver-config-buffer str))))
+    (set-process-sentinel
+     proc (lambda (proc event)
+            (if (string-equal event "finished\n")
+                (let ((weaver-config
+                       (ignore-errors
+                         (eval (car (read-from-string pod-weaver-config-buffer))))))
+                  (if weaver-config (pod-enable-weaver-features weaver-config))))
+            (setq pod-weaver-config-buffer "")))))
+
+(defun pod-add-support-for-weaver ()
+  (let ((project-root (ignore-errors (eproject-maybe-turn-on))))
+    (if project-root (pod-load-weaver-config project-root))))
+
 ;; main
 (defun pod-mode ()
   "Major mode for editing POD files (Plain Old Documentation for Perl)."
@@ -160,6 +199,7 @@
   (setq imenu-generic-expression '((nil "^=head[1234] +\\(.*\\)" 1)))
   (run-hooks 'pod-mode-hook)
   (pod-add-support-for-outline-minor-mode)
+  (pod-add-support-for-weaver)
   )
 
 (provide 'pod-mode)
